"""Etcd management tools."""

from typing import Any, Literal

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class EtcdMembersSchema(BaseModel):
    """Schema for etcd members arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")


class EtcdMembersTool(TalosTool):
    """Etcd members."""

    name = "talos_etcd_members"
    description = "List etcd members"
    args_schema = EtcdMembersSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = EtcdMembersSchema(**arguments)
        cmd = ["etcd", "members", "-n", args.nodes]
        return await self.execute_talosctl(cmd)


class EtcdSnapshotSchema(BaseModel):
    """Schema for etcd snapshot arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    path: str = Field(
        default="/tmp/etcd.snapshot",  # noqa: S108
        description="Path to save snapshot locally",
    )


class EtcdSnapshotTool(TalosTool):
    """Etcd snapshot."""

    name = "talos_etcd_snapshot"
    description = "Take an etcd snapshot"
    args_schema = EtcdSnapshotSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = EtcdSnapshotSchema(**arguments)
        cmd = ["etcd", "snapshot", args.path, "-n", args.nodes]
        return await self.execute_talosctl(cmd)


class EtcdAlarmSchema(BaseModel):
    """Schema for etcd alarm arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    action: Literal["list", "disarm"] = Field(
        default="list", description="Action: list, disarm"
    )


class EtcdAlarmTool(TalosTool):
    """Etcd alarms."""

    name = "talos_etcd_alarm"
    description = "List or disarm etcd alarms"
    args_schema = EtcdAlarmSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = EtcdAlarmSchema(**arguments)
        cmd = ["etcd", "alarm", args.action, "-n", args.nodes]
        return await self.execute_talosctl(cmd)


class EtcdDefragSchema(BaseModel):
    """Schema for etcd defrag arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")


class EtcdDefragTool(TalosTool):
    """Etcd defrag."""

    name = "talos_etcd_defrag"
    description = "Defragment etcd member"
    args_schema = EtcdDefragSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = EtcdDefragSchema(**arguments)
        cmd = ["etcd", "defrag", "-n", args.nodes]
        return await self.execute_talosctl(cmd)



"""Resource management tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class GetResourceSchema(BaseModel):
    """Schema for get resource arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    resource: str = Field(description="Resource type (e.g. members, services, machineconfig)")
    output: str = Field(default="yaml", description="Output format (yaml, json)")


class GetResourceTool(TalosTool):
    """Get any resource."""

    name = "talos_get"
    description = "Get a Talos resource definition"
    args_schema = GetResourceSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = GetResourceSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["get", args.resource, "-n", nodes, "-o", args.output]
        return await self.execute_talosctl(cmd)


class ListDefinitionsSchema(BaseModel):
    """Schema for list definitions arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")


class ListDefinitionsTool(TalosTool):
    """List generic definitions."""

    name = "talos_definitions"
    description = "List available resource definitions"
    args_schema = ListDefinitionsSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ListDefinitionsSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["get", "rd", "-n", nodes]
        return await self.execute_talosctl(cmd)


class GetVolumeStatusSchema(BaseModel):
    """Schema for volume status arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    volume: str = Field(description="Volume name (optional)", default="")
    output: str = Field(default="yaml", description="Output format (yaml, json)")


class GetVolumeStatusTool(TalosTool):
    """Get volume status."""

    name = "talos_volume_status"
    description = "Get volume status (PCRs for TPM encryption) - new in 1.12"
    args_schema = GetVolumeStatusSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = GetVolumeStatusSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["get", "volumestatus", "-n", nodes, "-o", args.output]
        if args.volume:
             cmd.insert(2, args.volume)
        return await self.execute_talosctl(cmd)


class GetKernelParamStatusSchema(BaseModel):
    """Schema for kernel param status arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    output: str = Field(default="yaml", description="Output format (yaml, json)")


class GetKernelParamStatusTool(TalosTool):
    """Get kernel param status."""

    name = "talos_kernel_param_status"
    description = "Get kernel param status (KSPP sysctl overrides) - new in 1.12"
    args_schema = GetKernelParamStatusSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = GetKernelParamStatusSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["get", "kernelparamstatus", "-n", nodes, "-o", args.output]
        return await self.execute_talosctl(cmd)



"""MCP Prompts implementation for Talos."""

from mcp.types import Prompt, PromptArgument, PromptMessage, TextContent

from talos_mcp.core.client import TalosClient


class TalosPrompts:
    """Talos Prompts handler."""

    def __init__(self, client: TalosClient) -> None:
        """Initialize TalosPrompts.

        Args:
            client: TalosClient instance
        """
        self.client = client

    async def list_prompts(self) -> list[Prompt]:
        """List available prompts."""
        return [
            Prompt(
                name="diagnose_cluster",
                description="Diagnose cluster health and issues",
                arguments=[
                    PromptArgument(
                        name="node",
                        description="Optional specific node IP to focus diagnosis on",
                        required=False,
                    )
                ],
            ),
            Prompt(
                name="audit_review",
                description="Review audit logs",
                arguments=[
                    PromptArgument(
                        name="limit",
                        description="Number of log lines to analyze (default 50)",
                        required=False,
                    )
                ],
            ),
        ]

    async def get_prompt(
        self, name: str, arguments: dict[str, str] | None = None
    ) -> list[PromptMessage]:
        """Get a prompt by name."""
        arguments = arguments or {}

        if name == "diagnose_cluster":
            node = arguments.get("node")
            node_context = f" for node {node}" if node else ""
            return [
                PromptMessage(
                    content=TextContent(
                        type="text",
                        text=f"""Please analyze the Talos cluster status{node_context}.
Run the following checks in order:
1. Check `talos_health` to see overall cluster state.
2. If there are unhealthy nodes, use `talos_service --node <IP>` to list failing services.
3. For any failing service, get logs using `talos_logs`.
4. Check `talos_get_stats` for resource usage.
5. Summarize the findings and recommend fixes.
""",
                    ),
                    role="user",
                )
            ]
        elif name == "audit_review":
            limit = int(arguments.get("limit", "50"))
            return [
                PromptMessage(
                    content=TextContent(
                        type="text",
                        text=f"""Please review the dashboard and audit logs.
1. Run `talos_dashboard` to get a snapshot of current activity.
2. Check `talos_dmesg` (last {limit} lines) for any kernel errors or warnings.
3. Summarize any potential security or stability issues found.
""",
                    ),
                    role="user",
                )
            ]
        else:
            raise ValueError(f"Unknown prompt: {name}")



"""Base classes for Talos MCP tools."""

from abc import ABC, abstractmethod
from typing import Any, ClassVar

from mcp.types import TextContent, Tool
from pydantic import BaseModel

from talos_mcp.core.client import TalosClient
from talos_mcp.core.exceptions import TalosCommandError


class TalosTool(ABC):
    """Base class for all Talos MCP tools."""

    name: ClassVar[str]
    description: ClassVar[str]
    args_schema: ClassVar[type[BaseModel]]  # Renamed from input_schema to be explicit

    def __init__(self, client: TalosClient) -> None:
        """Initialize the tool.

        Args:
            client: The TalosClient instance.
        """
        self.client = client

    def get_definition(self) -> Tool:
        """Get the MCP Tool definition.

        Returns:
            The Tool object.
        """
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema=self.args_schema.model_json_schema(),
        )

    @abstractmethod
    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Run the tool.

        Args:
            arguments: Tool arguments.

        Returns:
            List of TextContent results.
        """
        pass

    async def execute_talosctl(self, args: list[str]) -> list[TextContent]:
        """Helper to execute talosctl and return TextContent.

        Args:
            args: Arguments for talosctl.

        Returns:
            Formatted TextContent.
        """
        try:
            result = await self.client.execute_talosctl(args)
            output = result["stdout"]
            if result.get("stderr"):
                if output:
                    output += "\n\n"
                output += result["stderr"]
            return [TextContent(type="text", text=f"```\n{output}\n```")]
        except TalosCommandError as e:
            return [TextContent(type="text", text=f"Error executing {self.name}:\n{e.stderr}")]
        except Exception as e:
            return [TextContent(type="text", text=f"Error executing {self.name}:\n{e!s}")]

    def ensure_nodes(self, nodes: str | None) -> str:
        """Helper to ensure nodes are set, defaulting to all cluster nodes if None.
        
        Args:
            nodes: The provided nodes argument (comma-separated list or None).
            
        Returns:
            Comma-separated list of nodes.
        """
        if not nodes or nodes.lower() in ("all", "cluster"):
            all_nodes = self.client.get_nodes()
            return ",".join(all_nodes)
        return nodes



"""Custom exceptions for Talos MCP Server."""

class TalosError(Exception):
    """Base exception for all Talos MCP errors."""
    pass

class TalosConnectionError(TalosError):
    """Raised when unable to connect to a Talos node."""
    pass

class TalosCommandError(TalosError):
    """Raised when a talosctl command fails."""
    def __init__(self, cmd: list[str], returncode: int, stderr: str):
        self.cmd = cmd
        self.returncode = returncode
        self.stderr = stderr
        super().__init__(f"Command failed with code {returncode}: {stderr}")



import pytest

from talos_mcp.core.client import TalosClient
from talos_mcp.tools.files import ListFilesTool
from talos_mcp.tools.system import GetHealthTool, GetVersionTool


@pytest.mark.asyncio
async def test_version_integration():
    client = TalosClient()
    tool = GetVersionTool(client)
    results = await tool.run({})
    assert len(results) > 0
    assert "Tag" in results[0].text  # Should contain version info

@pytest.mark.asyncio
async def test_health_integration():
    client = TalosClient()
    tool = GetHealthTool(client)
    # Health checks might take a bit or return warnings on fresh clusters,
    # but the command should succeed (exit 0)
    results = await tool.run({})
    assert len(results) > 0
    # Output should not be an error
    assert "Error:" not in results[0].text

@pytest.mark.asyncio
async def test_ls_integration():
    client = TalosClient()
    tool = ListFilesTool(client)
    # List root
    results = await tool.run({"path": "/"})
    assert len(results) > 0
    assert "etc" in results[0].text
    assert "bin" in results[0].text



from unittest.mock import AsyncMock, MagicMock

import pytest

from talos_mcp.core.client import TalosClient


@pytest.fixture
def mock_talos_client():
    client = MagicMock(spec=TalosClient)
    client.execute_talosctl = AsyncMock()
    return client



"""Cluster lifecycle tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class RebootSchema(BaseModel):
    """Schema for reboot arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    mode: str = Field(default="default", description="Reboot mode: default, powercycle, force")


class RebootTool(TalosTool):
    """Reboot nodes."""

    name = "talos_reboot"
    description = "Reboot node(s)"
    args_schema = RebootSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = RebootSchema(**arguments)
        cmd = ["reboot", "-n", args.nodes]
        if args.mode != "default":
            cmd.extend(["--mode", args.mode])
        return await self.execute_talosctl(cmd)


class ShutdownSchema(BaseModel):
    """Schema for shutdown arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    force: bool = Field(default=False, description="Force shutdown")


class ShutdownTool(TalosTool):
    """Shutdown nodes."""

    name = "talos_shutdown"
    description = "Shutdown node(s)"
    args_schema = ShutdownSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ShutdownSchema(**arguments)
        cmd = ["shutdown", "-n", args.nodes]
        if args.force:
            cmd.append("--force")
        return await self.execute_talosctl(cmd)


class ResetSchema(BaseModel):
    """Schema for reset arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    reboot: bool = Field(default=False, description="Reboot after reset")
    system_labels_to_wipe: str = Field(
        default="", description="System labels to wipe"
    )
    graceful: bool = Field(default=True, description="Graceful reset")


class ResetTool(TalosTool):
    """Reset nodes."""

    name = "talos_reset"
    description = "Reset node(s) to maintenance mode or factory"
    args_schema = ResetSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ResetSchema(**arguments)
        cmd = ["reset", "-n", args.nodes]
        if args.reboot:
            cmd.append("--reboot")
        if args.system_labels_to_wipe:
            cmd.extend(["--system-labels-to-wipe", args.system_labels_to_wipe])
        if not args.graceful:
            cmd.extend(["--graceful=false"])
        return await self.execute_talosctl(cmd)


class UpgradeSchema(BaseModel):
    """Schema for upgrade arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    image: str = Field(description="Installer image to use")
    preserve: bool = Field(default=True, description="Preserve data")


class UpgradeTool(TalosTool):
    """Upgrade nodes."""

    name = "talos_upgrade"
    description = "Upgrade Talos on node(s)"
    args_schema = UpgradeSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = UpgradeSchema(**arguments)
        cmd = ["upgrade", "-n", args.nodes, "--image", args.image]
        if args.preserve:
            cmd.append("--preserve")
        return await self.execute_talosctl(cmd)


class ImageSchema(BaseModel):
    """Schema for image arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    cmd: str = Field(
        default="list",
        description="Command: list, pull, default, cache-create, cache-serve"
    )
    image: str = Field(default="", description="Image name (for pull/cache-create)")
    layout: str = Field(default="", description="Layout for cache commands (oci, flat)")
    platform: str = Field(default="", description="Platform for cache-create")


class ImageTool(TalosTool):
    """Manage images."""

    name = "talos_image"
    description = "Manage container images on Talos (new in 1.12)"
    args_schema = ImageSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ImageSchema(**arguments)

        base_cmd = ["image"]

        if args.cmd == "pull":
            if not args.image:
                return [TextContent(type="text", text="Image name required for pull")]
            base_cmd.extend(["pull", args.image])
        elif args.cmd == "default":
            base_cmd.append("default")
        elif args.cmd == "cache-create":
            if not args.image:
                return [TextContent(type="text", text="Image name required for cache-create")]
            base_cmd.extend(["cache-create", args.image])
            if args.layout:
                base_cmd.extend(["--layout", args.layout])
            if args.platform:
                base_cmd.extend(["--platform", args.platform])
        elif args.cmd == "cache-serve":
            base_cmd.append("cache-serve")
            if args.layout:
                base_cmd.extend(["--layout", args.layout])
        else:
            base_cmd.append("list")

        base_cmd.extend(["-n", args.nodes])

        return await self.execute_talosctl(base_cmd)


class BootstrapSchema(BaseModel):
    """Schema for bootstrap arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames (usually just one)")


class BootstrapTool(TalosTool):
    """Bootstrap cluster."""

    name = "talos_bootstrap"
    description = "Bootstrap the etcd cluster on the specified node"
    args_schema = BootstrapSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = BootstrapSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)

        # Bootstrap typically targets a single node
        # If multiple are provided, we should probably warn or just pass them (talosctl warns)
        cmd = ["bootstrap", "-n", nodes]
        return await self.execute_talosctl(cmd)


class ClusterShowSchema(BaseModel):
    """Schema for cluster show arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames (optional filter)")


class ClusterShowTool(TalosTool):
    """Show cluster status."""

    name = "talos_cluster_show"
    description = "High-level view of cluster members and their status"
    args_schema = ClusterShowSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ClusterShowSchema(**arguments)

        # cluster show doesn't require nodes usually (uses context)
        # but if nodes are provided, we can pass them
        cmd = ["cluster", "show"]
        if args.nodes:
            cmd.extend(["-n", args.nodes])

        return await self.execute_talosctl(cmd)



"""Tests for server module."""

import subprocess
import sys
from unittest.mock import Mock, patch

import pytest


class TestServerCLI:
    """Tests for CLI functionality."""

    def test_version_flag(self) -> None:
        """Test --version flag returns version and exits."""
        result = subprocess.run(
            [sys.executable, "-m", "talos_mcp.server", "--version"],
            capture_output=True,
            text=True,
            timeout=10,
        )
        assert result.returncode == 0
        assert "talos-mcp-server" in result.stdout
        assert "0.3" in result.stdout  # Version starts with 0.3.x

    def test_help_flag(self) -> None:
        """Test --help flag shows usage information."""
        import re
        result = subprocess.run(
            [sys.executable, "-m", "talos_mcp.server", "--help"],
            capture_output=True,
            text=True,
            timeout=10,
        )
        # Strip ANSI escape codes
        output = re.sub(r'\x1b\[[0-9;]*m', '', result.stdout)
        assert result.returncode == 0
        assert "Talos MCP Server" in output
        assert "log" in output.lower()  # --log-level option
        assert "readonly" in output.lower()

    def test_invalid_option(self) -> None:
        """Test invalid option returns error."""
        result = subprocess.run(
            [sys.executable, "-m", "talos_mcp.server", "--invalid-option"],
            capture_output=True,
            text=True,
            timeout=10,
        )
        assert result.returncode != 0
        assert "No such option" in result.stderr


class TestServerImports:
    """Tests for module imports and configuration."""

    def test_server_module_imports(self) -> None:
        """Test that server module imports without errors."""
        from talos_mcp import server
        assert hasattr(server, "cli")
        assert hasattr(server, "main")
        assert hasattr(server, "__version__")

    def test_version_is_valid_semver(self) -> None:
        """Test that version follows semver pattern."""
        from talos_mcp.server import __version__
        parts = __version__.split(".")
        assert len(parts) >= 2
        assert all(p.isdigit() for p in parts[:2])

    def test_settings_module_imports(self) -> None:
        """Test that settings module imports correctly."""
        from talos_mcp.core.settings import Settings, settings
        assert isinstance(settings, Settings)
        assert hasattr(settings, "log_level")
        assert hasattr(settings, "readonly")

    def test_tools_registration(self) -> None:
        """Test that all tool modules import correctly."""
        from talos_mcp.tools import (
            base,
            cluster,
            config,
            etcd,
            files,
            network,
            resources,
            services,
            system,
        )
        # All modules should have classes inheriting from TalosTool
        assert hasattr(base, "TalosTool")
        assert hasattr(cluster, "RebootTool")
        assert hasattr(config, "ApplyTool")
        assert hasattr(etcd, "EtcdMembersTool")
        assert hasattr(files, "ReadFileTool")
        assert hasattr(network, "InterfacesTool")
        assert hasattr(resources, "GetResourceTool")
        assert hasattr(services, "ServiceTool")
        assert hasattr(system, "GetVersionTool")


class TestReadonlyMode:
    """Tests for readonly mode enforcement."""

    def test_write_tools_defined(self) -> None:
        """Test that WRITE_TOOLS set is defined."""
        from talos_mcp.server import WRITE_TOOLS
        assert isinstance(WRITE_TOOLS, set)
        assert len(WRITE_TOOLS) > 0
        # Should contain known write operations
        assert "talos_reboot" in WRITE_TOOLS
        assert "talos_shutdown" in WRITE_TOOLS
        assert "talos_apply" in WRITE_TOOLS

    def test_readonly_setting_default(self) -> None:
        """Test that readonly defaults to False."""
        from talos_mcp.core.settings import Settings
        fresh_settings = Settings()
        assert fresh_settings.readonly is False



"""Network tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class InterfacesSchema(BaseModel):
    """Schema for interfaces arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")


class InterfacesTool(TalosTool):
    """List interfaces."""

    name = "talos_interfaces"
    description = "List network interfaces"
    args_schema = InterfacesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = InterfacesSchema(**arguments)
        # talosctl get links ?? Or specific command?
        # `talosctl get links` or `talosctl interfaces` (deprecated?)
        # Let's use `get addresses` which is common for "interfaces" alias in my analysis
        return await self.execute_talosctl(["get", "addresses", "-n", args.nodes])


class RoutesSchema(BaseModel):
    """Schema for routes arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")


class RoutesTool(TalosTool):
    """List routes."""

    name = "talos_routes"
    description = "List routing table"
    args_schema = RoutesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = RoutesSchema(**arguments)
        cmd = ["get", "routes", "-n", args.nodes]
        return await self.execute_talosctl(cmd)


class NetstatSchema(BaseModel):
    """Schema for netstat arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")


class NetstatTool(TalosTool):
    """Netstat."""

    name = "talos_netstat"
    description = "List network connections"
    args_schema = NetstatSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NetstatSchema(**arguments)
        cmd = ["netstat", "-n", args.nodes]
        return await self.execute_talosctl(cmd)


class PcapSchema(BaseModel):
    """Schema for pcap arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    interface: str = Field(description="Interface to capture on")
    duration: str = Field(default="10s", description="Duration to capture")


class PcapTool(TalosTool):
    """Paket capture."""

    name = "talos_pcap"
    description = "Capture packets (stub)"
    args_schema = PcapSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        _ = PcapSchema(**arguments)
        # Stub implementation as binary output handling is complex
        return [
            TextContent(
                type="text",
                text="Packet capture not fully implemented (requires binary streaming)",
            )
        ]



"""Tests for prompts module."""

import pytest
from unittest.mock import Mock, AsyncMock

from talos_mcp.prompts import TalosPrompts
from talos_mcp.core.client import TalosClient


@pytest.fixture
def mock_client() -> Mock:
    """Create a mock TalosClient."""
    client = Mock(spec=TalosClient)
    return client


@pytest.fixture
def prompts(mock_client: Mock) -> TalosPrompts:
    """Create TalosPrompts instance with mocked client."""
    return TalosPrompts(mock_client)


class TestTalosPrompts:
    """Tests for TalosPrompts class."""

    async def test_list_prompts(self, prompts: TalosPrompts) -> None:
        """Test list_prompts returns expected prompts."""
        result = await prompts.list_prompts()
        assert len(result) == 2
        
        names = [p.name for p in result]
        assert "diagnose_cluster" in names
        assert "audit_review" in names

    async def test_list_prompts_has_arguments(self, prompts: TalosPrompts) -> None:
        """Test prompts have proper arguments defined."""
        result = await prompts.list_prompts()
        
        diagnose = next(p for p in result if p.name == "diagnose_cluster")
        assert len(diagnose.arguments) == 1
        assert diagnose.arguments[0].name == "node"
        assert diagnose.arguments[0].required is False

    async def test_get_prompt_diagnose_cluster(self, prompts: TalosPrompts) -> None:
        """Test get_prompt for diagnose_cluster."""
        result = await prompts.get_prompt("diagnose_cluster")
        assert len(result) == 1
        assert result[0].role == "user"
        assert "talos_health" in result[0].content.text

    async def test_get_prompt_diagnose_cluster_with_node(self, prompts: TalosPrompts) -> None:
        """Test get_prompt for diagnose_cluster with specific node."""
        result = await prompts.get_prompt("diagnose_cluster", {"node": "10.0.0.1"})
        assert "for node 10.0.0.1" in result[0].content.text

    async def test_get_prompt_audit_review(self, prompts: TalosPrompts) -> None:
        """Test get_prompt for audit_review."""
        result = await prompts.get_prompt("audit_review")
        assert len(result) == 1
        assert "talos_dashboard" in result[0].content.text
        assert "50 lines" in result[0].content.text

    async def test_get_prompt_audit_review_with_limit(self, prompts: TalosPrompts) -> None:
        """Test get_prompt for audit_review with custom limit."""
        result = await prompts.get_prompt("audit_review", {"limit": "100"})
        assert "100 lines" in result[0].content.text

    async def test_get_prompt_unknown_raises(self, prompts: TalosPrompts) -> None:
        """Test get_prompt raises for unknown prompt name."""
        with pytest.raises(ValueError, match="Unknown prompt"):
            await prompts.get_prompt("nonexistent_prompt")



import asyncio
import sys
import time

from talos_mcp.core.client import TalosClient
from talos_mcp.tools.system import GetVersionTool


async def wait_for_cluster(timeout: int = 300):
    print(f"Waiting for cluster to be ready (timeout {timeout}s)...")
    start = time.time()
    client = TalosClient()
    tool = GetVersionTool(client)

    while True:
        try:
            # Try to get version
            results = await tool.run({})
            if results and "Tag" in results[0].text:
                print("Cluster is ready!")
                return
        except Exception:  # noqa: S110
            pass  # Cluster not ready yet, retry

        if time.time() - start > timeout:
            raise TimeoutError("Timed out waiting for cluster to be ready")

        await asyncio.sleep(2)

if __name__ == "__main__":
    try:
        asyncio.run(wait_for_cluster())
    except Exception as e:
        print(f"Error: {e}")
        sys.exit(1)



"""Configuration tools."""

import json
from typing import Any, Literal

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class ConfigInfoSchema(BaseModel):
    """Schema for config info arguments."""

    pass


class ConfigInfoTool(TalosTool):
    """Get config info."""

    name = "talos_config_info"
    description = "Get information about the current Talos configuration context"
    args_schema = ConfigInfoSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        # arguments is unused but required by signature
        _ = arguments
        info = self.client.get_context_info()
        return [TextContent(type="text", text=json.dumps(info, indent=2))]


class KubeconfigSchema(BaseModel):
    """Schema for kubeconfig arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    force: bool = Field(default=False, description="Force overwrite")


class GetKubeconfigTool(TalosTool):
    """Get kubeconfig."""

    name = "talos_kubeconfig"
    description = "Retrieve kubeconfig from the cluster"
    args_schema = KubeconfigSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = KubeconfigSchema(**arguments)
        cmd = ["kubeconfig", "-n", args.nodes]
        if args.force:
            cmd.append("--force")
        return await self.execute_talosctl(cmd)


class ApplyConfigSchema(BaseModel):
    """Schema for apply config arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    file: str = Field(description="Path to config file")
    mode: str = Field(default="auto", description="Mode: auto, reboot, no-reboot")


class ApplyConfigTool(TalosTool):
    """Apply configuration."""

    name = "talos_apply_config"
    description = "Apply a new configuration to node(s) - Deprecated in 1.12, use talos_apply"
    args_schema = ApplyConfigSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ApplyConfigSchema(**arguments)
        cmd = ["apply-config", "-f", args.file, "-n", args.nodes, "--mode", args.mode]
        return await self.execute_talosctl(cmd)


class ApplySchema(BaseModel):
    """Schema for generic apply arguments."""

    nodes: str = Field(description="Comma-separated list of node IPs/hostnames")
    file: str = Field(description="Path to manifest file")
    mode: str = Field(default="auto", description="Mode: auto, reboot, no-reboot")


class ApplyTool(TalosTool):
    """Apply generic manifest."""

    name = "talos_apply"
    description = "Apply a manifest to node(s) (new in 1.12)"
    args_schema = ApplySchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ApplySchema(**arguments)
        cmd = ["apply", "-f", args.file, "-n", args.nodes, "--mode", args.mode]
        return await self.execute_talosctl(cmd)


class ValidateConfigSchema(BaseModel):
    """Schema for validate config arguments."""

    file: str = Field(description="Path to config file")
    mode: Literal["metal", "cloud", "container"] = Field(
        default="metal", description="Validation mode"
    )


class ValidateConfigTool(TalosTool):
    """Validate configuration."""

    name = "talos_validate_config"
    description = "Validate a Talos configuration file"
    args_schema = ValidateConfigSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ValidateConfigSchema(**arguments)
        cmd = ["validate", "-c", args.file, "--mode", args.mode]
        return await self.execute_talosctl(cmd)


class PatchSchema(BaseModel):
    """Schema for patch arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames")
    type: str = Field(description="Resource type (e.g., MachineConfig, Service)")
    id: str | None = Field(default=None, description="Resource ID")
    patch: str = Field(description="JSON or YAML patch content")
    mode: Literal["strategic", "json"] = Field(default="strategic", description="Patch mode")


class PatchTool(TalosTool):
    """Patch resource."""

    name = "talos_patch"
    description = "Apply a patch to a specific resource"
    args_schema = PatchSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = PatchSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)

        cmd = ["patch", args.type]
        if args.id:
            cmd.append(args.id)

        cmd.extend(["-n", nodes, "--patch", args.patch, "--mode", args.mode])
        return await self.execute_talosctl(cmd)


class MachineConfigPatchSchema(BaseModel):
    """Schema for machineconfig patch arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames")
    patch: str = Field(description="YAML patch content")
    mode: Literal["auto", "reboot", "no-reboot"] = Field(default="auto", description="Apply mode")


class MachineConfigPatchTool(TalosTool):
    """Patch machineconfig."""

    name = "talos_machineconfig_patch"
    description = "Patch the machine configuration directly"
    args_schema = MachineConfigPatchSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = MachineConfigPatchSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)

        # We need to pipe the patch content to stdin since talosctl expects a file or stdin
        # constructing command with --patch is supported in newer talosctl but stdin is safer for complex yaml
        # Actually checking CLI help: `talosctl machineconfig patch [flags]` read from stdin if no file
        # But we can also use `--patch` flag if supported.
        # Standard way is `talosctl machineconfig patch --patch <content> -n <node>`

        cmd = ["machineconfig", "patch", "--patch", args.patch, "-n", nodes, "--mode", args.mode]
        return await self.execute_talosctl(cmd)


class GenConfigSchema(BaseModel):
    """Schema for gen config arguments."""

    name: str = Field(description="Cluster name")
    endpoint: str = Field(description="Cluster endpoint (https://VIP:6443)")
    output_dir: str = Field(default="./", description="Output directory")
    version: str | None = Field(default=None, description="Kubernetes version")


class GenConfigTool(TalosTool):
    """Generate configuration."""

    name = "talos_gen_config"
    description = "Generate a new cluster configuration (local operation)"
    args_schema = GenConfigSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = GenConfigSchema(**arguments)
        cmd = ["gen", "config", args.name, args.endpoint, "--output-dir", args.output_dir]
        if args.version:
            cmd.extend(["--kubernetes-version", args.version])

        # This runs locally, does not require nodes
        return await self.execute_talosctl(cmd)




import pytest

from talos_mcp.core.client import TalosClient
from talos_mcp.tools.services import ServiceTool


@pytest.mark.asyncio
async def test_service_restart_integration():
    """Test a Write operation: Restarting a service."""
    client = TalosClient()
    tool = ServiceTool(client)

    # Restart kubelet (safe for test cluster)
    # We use 'restart' action on 'kubelet'
    results = await tool.run({
        "action": "restart",
        "service": "kubelet"
    })

    # talosctl service restart usually returns empty output on success,
    # or just doesn't error out.
    assert len(results) > 0
    # On success, it might print confirmation or be empty.
    # If it failed, it would raise or return error text.
    assert "Error" not in results[0].text



import pytest

from talos_mcp.tools.cluster import BootstrapTool, ClusterShowTool
from talos_mcp.tools.config import GenConfigTool, MachineConfigPatchTool, PatchTool
from talos_mcp.tools.system import DevicesTool, DisksTool


@pytest.mark.asyncio
async def test_patch_tool(mock_talos_client):
    mock_talos_client.get_nodes.return_value = ["10.0.0.1", "10.0.0.2"]
    mock_talos_client.execute_talosctl.return_value = {"stdout": "patched", "stderr": ""}
    tool = PatchTool(mock_talos_client)
    # Test valid patch
    args = {
        "nodes": "10.0.0.1",
        "type": "MachineConfig",
        "patch": '{"machine": {"type": "worker"}}',
        "mode": "strategic"
    }
    await tool.run(args)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["patch", "MachineConfig", "-n", "10.0.0.1", "--patch", '{"machine": {"type": "worker"}}', "--mode", "strategic"]
    )

@pytest.mark.asyncio
async def test_machineconfig_patch_tool(mock_talos_client):
    mock_talos_client.get_nodes.return_value = ["10.0.0.1", "10.0.0.2"]
    mock_talos_client.execute_talosctl.return_value = {"stdout": "patched", "stderr": ""}
    tool = MachineConfigPatchTool(mock_talos_client)
    args = {
        "nodes": "10.0.0.1",
        "patch": "machine:\n  type: worker",
        "mode": "no-reboot"
    }
    await tool.run(args)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["machineconfig", "patch", "--patch", "machine:\n  type: worker", "-n", "10.0.0.1", "--mode", "no-reboot"]
    )

@pytest.mark.asyncio
async def test_gen_config_tool(mock_talos_client):
    mock_talos_client.execute_talosctl.return_value = {"stdout": "generated", "stderr": ""}
    tool = GenConfigTool(mock_talos_client)
    args = {
        "name": "test-cluster",
        "endpoint": "https://1.1.1.1:6443",
        "output_dir": "./out",
        "version": "v1.30.0"
    }
    await tool.run(args)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["gen", "config", "test-cluster", "https://1.1.1.1:6443", "--output-dir", "./out", "--kubernetes-version", "v1.30.0"]
    )

@pytest.mark.asyncio
async def test_bootstrap_tool(mock_talos_client):
    mock_talos_client.get_nodes.return_value = ["10.0.0.1", "10.0.0.2"]
    mock_talos_client.execute_talosctl.return_value = {"stdout": "bootstrapped", "stderr": ""}
    tool = BootstrapTool(mock_talos_client)
    args = {"nodes": "10.0.0.1"}
    await tool.run(args)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["bootstrap", "-n", "10.0.0.1"]
    )

@pytest.mark.asyncio
async def test_cluster_show_tool(mock_talos_client):
    mock_talos_client.execute_talosctl.return_value = {"stdout": "cluster status", "stderr": ""}
    tool = ClusterShowTool(mock_talos_client)
    args = {}
    await tool.run(args)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["cluster", "show"]
    )

    # Test with nodes arg
    args_with_nodes = {"nodes": "10.0.0.1"}
    await tool.run(args_with_nodes)
    mock_talos_client.execute_talosctl.assert_called_with(
        ["cluster", "show", "-n", "10.0.0.1"]
    )

@pytest.mark.asyncio
async def test_disks_tool(mock_talos_client):
    mock_talos_client.get_nodes.return_value = ["10.0.0.1", "10.0.0.2"]
    mock_talos_client.execute_talosctl.return_value = {"stdout": "disks", "stderr": ""}
    tool = DisksTool(mock_talos_client)
    await tool.run({})
    mock_talos_client.execute_talosctl.assert_called_with(
        ["get", "disks", "-n", "10.0.0.1,10.0.0.2"]  # Default all nodes from mock
    )

@pytest.mark.asyncio
async def test_devices_tool(mock_talos_client):
    mock_talos_client.get_nodes.return_value = ["10.0.0.1", "10.0.0.2"]
    mock_talos_client.execute_talosctl.return_value = {"stdout": "devices", "stderr": ""}
    tool = DevicesTool(mock_talos_client)
    await tool.run({})
    mock_talos_client.execute_talosctl.assert_called_with(
        ["get", "devices", "-n", "10.0.0.1,10.0.0.2"]
    )



import pytest
from pydantic import BaseModel, Field

from talos_mcp.core.exceptions import TalosCommandError
from talos_mcp.tools.base import TalosTool


class SchemaForTest(BaseModel):
    arg1: str = Field(description="Test argument")

class ToolForTest(TalosTool):
    name = "test_tool"
    description = "A test tool"
    args_schema = SchemaForTest

    async def run(self, arguments: dict):
        args = SchemaForTest(**arguments)
        return await self.execute_talosctl(["echo", args.arg1])

@pytest.mark.asyncio
async def test_talos_tool_definition(mock_talos_client):
    tool = ToolForTest(mock_talos_client)
    definition = tool.get_definition()
    assert definition.name == "test_tool"
    assert definition.description == "A test tool"
    assert "arg1" in definition.inputSchema["properties"]

@pytest.mark.parametrize(
    "name, mock_response, mock_side_effect, inputs, expected_output_contains, expected_error",
    [
        (
            "Success",
            {"stdout": "success output", "stderr": ""},
            None,
            {"arg1": "value"},
            "success output",
            None,
        ),
        (
            "Success with stderr (Health check style)",
            {"stdout": "standard output", "stderr": "warning message"},
            None,
            {"arg1": "value"},
            "warning message",
            None,
        ),
        (
            "Failure",
            None,
            TalosCommandError(cmd=["echo", "value"], returncode=1, stderr="error message"),
            {"arg1": "value"},
            "error message",
            "Error executing test_tool",
        ),
    ],
)
@pytest.mark.asyncio
async def test_talos_tool_execution(
    mock_talos_client,
    name,
    mock_response,
    mock_side_effect,
    inputs,
    expected_output_contains,
    expected_error,
):
    """Table-driven test for TalosTool execution."""
    if mock_side_effect:
        mock_talos_client.execute_talosctl.side_effect = mock_side_effect
    else:
        mock_talos_client.execute_talosctl.return_value = mock_response

    tool = ToolForTest(mock_talos_client)
    result = await tool.run(inputs)

    assert len(result) == 1
    text = result[0].text

    if expected_error:
        assert expected_error in text
    else:
        assert text.startswith("```\n")

    assert expected_output_contains in text



"""Tests for resources module."""

import pytest
from unittest.mock import Mock, AsyncMock, patch

from pydantic import AnyUrl

from talos_mcp.resources import TalosResources
from talos_mcp.core.client import TalosClient
from talos_mcp.core.exceptions import TalosCommandError


@pytest.fixture
def mock_client() -> Mock:
    """Create a mock TalosClient."""
    client = Mock(spec=TalosClient)
    client.execute_talosctl = AsyncMock()
    client.get_context_info = Mock(return_value={"context": "test", "endpoints": ["10.0.0.1"]})
    return client


@pytest.fixture
def resources(mock_client: Mock) -> TalosResources:
    """Create TalosResources instance with mocked client."""
    return TalosResources(mock_client)


class TestTalosResources:
    """Tests for TalosResources class."""

    async def test_list_resources_empty(self, resources: TalosResources) -> None:
        """Test list_resources returns empty list (dynamic resources use templates)."""
        result = await resources.list_resources()
        assert result == []

    async def test_list_resource_templates(self, resources: TalosResources) -> None:
        """Test list_resource_templates returns expected templates."""
        result = await resources.list_resource_templates()
        assert len(result) == 3
        
        uris = [t.uriTemplate for t in result]
        assert "talos://{node}/health" in uris
        assert "talos://{node}/version" in uris
        assert "talos://{node}/config" in uris

    async def test_list_resource_templates_have_names(self, resources: TalosResources) -> None:
        """Test resource templates have proper names."""
        result = await resources.list_resource_templates()
        names = [t.name for t in result]
        assert "Node Health" in names
        assert "Node Version" in names
        assert "Node Config Context" in names


class TestReadResource:
    """Tests for read_resource method."""

    async def test_read_resource_health(
        self, resources: TalosResources, mock_client: Mock
    ) -> None:
        """Test reading health resource."""
        mock_client.execute_talosctl.return_value = {"stdout": "OK", "stderr": ""}
        
        uri = AnyUrl("talos://10.0.0.1/health")
        result = await resources.read_resource(uri)
        
        assert result == "OK"
        mock_client.execute_talosctl.assert_called_once()
        call_args = mock_client.execute_talosctl.call_args[0][0]
        assert "health" in call_args
        assert "--nodes" in call_args
        assert "10.0.0.1" in call_args

    async def test_read_resource_version(
        self, resources: TalosResources, mock_client: Mock
    ) -> None:
        """Test reading version resource."""
        mock_client.execute_talosctl.return_value = {"stdout": "v1.12.0", "stderr": ""}
        
        uri = AnyUrl("talos://10.0.0.1/version")
        result = await resources.read_resource(uri)
        
        assert "v1.12.0" in result

    async def test_read_resource_config(
        self, resources: TalosResources, mock_client: Mock
    ) -> None:
        """Test reading config resource."""
        uri = AnyUrl("talos://10.0.0.1/config")
        result = await resources.read_resource(uri)
        
        assert "context" in result
        assert "test" in result

    async def test_read_resource_unsupported_scheme(
        self, resources: TalosResources
    ) -> None:
        """Test reading resource with unsupported scheme raises."""
        uri = AnyUrl("http://10.0.0.1/health")
        with pytest.raises(ValueError, match="Unsupported scheme"):
            await resources.read_resource(uri)

    async def test_read_resource_unknown_type(
        self, resources: TalosResources
    ) -> None:
        """Test reading unknown resource type raises."""
        uri = AnyUrl("talos://10.0.0.1/unknown")
        with pytest.raises(ValueError, match="Unknown resource type"):
            await resources.read_resource(uri)

    async def test_read_resource_health_error(
        self, resources: TalosResources, mock_client: Mock
    ) -> None:
        """Test reading health with TalosCommandError returns error message."""
        mock_client.execute_talosctl.side_effect = TalosCommandError(
            1, "health", "connection refused"
        )
        
        uri = AnyUrl("talos://10.0.0.1/health")
        result = await resources.read_resource(uri)
        
        assert "Error" in result
        assert "connection refused" in result

    async def test_read_resource_version_generic_error(
        self, resources: TalosResources, mock_client: Mock
    ) -> None:
        """Test reading version with generic exception returns error message."""
        mock_client.execute_talosctl.side_effect = Exception("Network error")
        
        uri = AnyUrl("talos://10.0.0.1/version")
        result = await resources.read_resource(uri)
        
        assert "Error" in result
        assert "Network error" in result



"""File management tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class ListFilesSchema(BaseModel):
    """Schema for list files arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    path: str = Field(default="/", description="Directory path")


class ListFilesTool(TalosTool):
    """List files on a node."""

    name = "talos_ls"
    description = "List files and directories"
    args_schema = ListFilesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ListFilesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["ls", args.path, "-n", nodes]
        return await self.execute_talosctl(cmd)


class ReadFileSchema(BaseModel):
    """Schema for read file arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    path: str = Field(description="File path to read")


class ReadFileTool(TalosTool):
    """Read a file."""

    name = "talos_cat"
    description = "Read file content"
    args_schema = ReadFileSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ReadFileSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["read", args.path, "-n", nodes]
        return await self.execute_talosctl(cmd)


class CopySchema(BaseModel):
    """Schema for copy arguments."""

    # Copying to/from multiple nodes is complex. `talosctl cp` might not support it for download.
    # Upload to multiple nodes works.
    # Let's support it, but behavior depends on talosctl.
    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    src: str = Field(description="Source path")
    dst: str = Field(description="Destination path")
    direction: str = Field(
        default="download",
        description="Direction: upload (local->node) or download (node->local)",
    )


class CopyTool(TalosTool):
    """Copy files."""

    name = "talos_cp"
    description = "Copy files to/from node"
    args_schema = CopySchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = CopySchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)

        # talosctl cp requires -n <node> even if the target is specified as <node>:<path>
        # to ensure it knows which context/auth to use for that node IP.

        if args.direction == "upload":
            cmd = ["cp", args.src, f"{nodes}:{args.dst}"]
        else:
            cmd = ["cp", f"{nodes}:{args.src}", args.dst]

        cmd.extend(["-n", nodes])

        return await self.execute_talosctl(cmd)


class DiskUsageSchema(BaseModel):
    """Schema for disk usage arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    path: str = Field(default="/", description="Path to check")


class DiskUsageTool(TalosTool):
    """Disk Usage."""

    name = "talos_du"
    description = "Check disk usage"
    args_schema = DiskUsageSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = DiskUsageSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["usage", args.path, "-n", nodes]
        return await self.execute_talosctl(cmd)


class MountsSchema(BaseModel):
    """Schema for mounts arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")


class MountsTool(TalosTool):
    """List mounts."""

    name = "talos_mounts"
    description = "List mount points"
    args_schema = MountsSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = MountsSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["mounts", "-n", nodes]
        return await self.execute_talosctl(cmd)



"""Talos MCP Server."""

import asyncio
import signal
import sys
from typing import Any

import typer
import anyio
import uvloop
from loguru import logger
from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.types import (
    GetPromptResult,
    Prompt,
    Resource,
    ResourceTemplate,
    TextContent,
    Tool,
)
from pydantic import AnyUrl

from talos_mcp.core.client import TalosClient
from talos_mcp.core.settings import settings
from talos_mcp.prompts import TalosPrompts
from talos_mcp.resources import TalosResources
from talos_mcp.tools.cluster import (
    BootstrapTool,
    ClusterShowTool,
    ImageTool,
    RebootTool,
    ResetTool,
    ShutdownTool,
    UpgradeTool,
)
from talos_mcp.tools.config import (
    ApplyConfigTool,
    ApplyTool,
    ConfigInfoTool,
    GenConfigTool,
    GetKubeconfigTool,
    MachineConfigPatchTool,
    PatchTool,
    ValidateConfigTool,
)
from talos_mcp.tools.etcd import (
    EtcdAlarmTool,
    EtcdDefragTool,
    EtcdMembersTool,
    EtcdSnapshotTool,
)
from talos_mcp.tools.files import (
    CopyTool,
    DiskUsageTool,
    ListFilesTool,
    MountsTool,
    ReadFileTool,
)
from talos_mcp.tools.network import (
    InterfacesTool,
    NetstatTool,
    PcapTool,
    RoutesTool,
)
from talos_mcp.tools.resources import (
    GetKernelParamStatusTool,
    GetResourceTool,
    GetVolumeStatusTool,
    ListDefinitionsTool,
)
from talos_mcp.tools.services import (
    DmesgTool,
    EventsTool,
    LogsTool,
    ServiceTool,
)
from talos_mcp.tools.system import (
    DashboardTool,
    DevicesTool,
    DisksTool,
    GetContainersTool,
    GetHealthTool,
    GetProcessesTool,
    GetStatsTool,
    GetVersionTool,
    MemoryTool,
    TimeTool,
)


# Configure logging
def configure_logging() -> None:
    """Configure logging with detailed formatting and auditing.
    
    Uses settings from Settings class for all configuration.
    """
    logger.remove()  # Remove default handler

    # Standard stderr logging
    logger.add(
        sys.stderr,
        format=settings.log_format,
        level=settings.log_level.upper(),
    )

    # Audit log to file
    logger.add(
        settings.audit_log_path,
        rotation=settings.audit_log_rotation,
        retention=settings.audit_log_retention,
        level="DEBUG",
        format="{time:YYYY-MM-DD HH:mm:ss.SSS} | {level} | {name}:{function}:{line} | {message} | {extra}",
        serialize=settings.audit_log_serialize,
    )



# Initialize the MCP server
app_mcp = Server("talos-mcp-server")
talos_client = TalosClient()
talos_resources = TalosResources(talos_client)
talos_prompts = TalosPrompts(talos_client)


# Register Resources Handlers
@app_mcp.list_resources()  # type: ignore
async def list_resources() -> list[Resource]:
    """List available resources."""
    return await talos_resources.list_resources()


@app_mcp.list_resource_templates()  # type: ignore
async def list_resource_templates() -> list[ResourceTemplate]:
    """List available resource templates."""
    return await talos_resources.list_resource_templates()


@app_mcp.read_resource()  # type: ignore
async def read_resource(uri: AnyUrl) -> str | bytes:
    """Read a resource."""
    return await talos_resources.read_resource(uri)


# Register Prompts Handlers
@app_mcp.list_prompts()  # type: ignore
async def list_prompts() -> list[Prompt]:
    """List available prompts."""
    return await talos_prompts.list_prompts()


@app_mcp.get_prompt()  # type: ignore
async def get_prompt(name: str, arguments: dict[str, str] | None = None) -> GetPromptResult:
    """Get a prompt by name."""
    messages = await talos_prompts.get_prompt(name, arguments)
    return GetPromptResult(messages=messages)

# Register tools
tools_list = [
    # System
    GetVersionTool(talos_client),
    GetHealthTool(talos_client),
    GetStatsTool(talos_client),
    GetContainersTool(talos_client),
    GetProcessesTool(talos_client),
    DashboardTool(talos_client),
    MemoryTool(talos_client),
    TimeTool(talos_client),
    DisksTool(talos_client),
    DevicesTool(talos_client),
    # Files
    ListFilesTool(talos_client),
    ReadFileTool(talos_client),
    CopyTool(talos_client),
    DiskUsageTool(talos_client),
    MountsTool(talos_client),
    # Network
    InterfacesTool(talos_client),
    RoutesTool(talos_client),
    NetstatTool(talos_client),
    PcapTool(talos_client),
    # Services
    ServiceTool(talos_client),
    LogsTool(talos_client),
    DmesgTool(talos_client),
    EventsTool(talos_client),
    # Cluster
    RebootTool(talos_client),
    ShutdownTool(talos_client),
    ResetTool(talos_client),
    UpgradeTool(talos_client),
    ImageTool(talos_client),
    BootstrapTool(talos_client),
    ClusterShowTool(talos_client),
    # Etcd
    EtcdMembersTool(talos_client),
    EtcdSnapshotTool(talos_client),
    EtcdAlarmTool(talos_client),
    EtcdDefragTool(talos_client),
    # Config
    ConfigInfoTool(talos_client),
    GetKubeconfigTool(talos_client),
    ApplyConfigTool(talos_client),
    ApplyTool(talos_client),
    ValidateConfigTool(talos_client),
    PatchTool(talos_client),
    MachineConfigPatchTool(talos_client),
    GenConfigTool(talos_client),
    # Resources
    GetResourceTool(talos_client),
    ListDefinitionsTool(talos_client),
    GetVolumeStatusTool(talos_client),
    GetKernelParamStatusTool(talos_client),
]

tools_map = {tool.name: tool for tool in tools_list}

# Tools that perform write/mutating operations
WRITE_TOOLS = {
    "talos_reboot",
    "talos_shutdown",
    "talos_reset",
    "talos_upgrade",
    "talos_apply_config",
    "talos_apply",
    "talos_patch",
    "talos_machineconfig_patch",
    "talos_bootstrap",
    "talos_etcd_defrag",
    "talos_etcd_alarm",
    "talos_cp",
}


@app_mcp.list_tools()  # type: ignore
async def list_tools() -> list[Tool]:
    """List all available Talos tools."""
    return [tool.get_definition() for tool in tools_list]


@app_mcp.call_tool()
async def call_tool(name: str, arguments: Any) -> list[TextContent]:
    """Handle tool calls for Talos operations."""
    # Enforce read-only mode
    if settings.readonly and name in WRITE_TOOLS:
        return [
            TextContent(
                type="text",
                text=f"Error: Tool '{name}' is blocked in read-only mode. "
                "Set TALOS_MCP_READONLY=false or remove --readonly flag to enable.",
            )
        ]

    tool = tools_map.get(name)
    if not tool:
        return [TextContent(type="text", text=f"Unknown tool: {name}")]

    try:
        if not isinstance(arguments, dict):
            # Ensure arguments is a dict, MCP sometimes sends generic object?
            # Type hint says Any, but typically it's a dict.
            # If it's None, create empty dict.
            arguments = arguments or {}

        return await tool.run(arguments)
    except Exception as e:
        logger.error(f"Error executing tool {name}: {e}")
        return [TextContent(type="text", text=f"Error: {e!s}")]


__version__ = "0.3.8"


def version_callback(value: bool) -> None:
    """Show version and exit."""
    if value:
        typer.echo(f"talos-mcp-server {__version__}")
        raise typer.Exit()


cli = typer.Typer()


@cli.command()
def main(
    version: bool = typer.Option(
        False, "--version", "-V", callback=version_callback, is_eager=True,
        help="Show version and exit"
    ),
    log_level: str = typer.Option(
        "INFO", "--log-level", "-l", help="Log level (DEBUG, INFO, WARNING, ERROR)",
        envvar="TALOS_MCP_LOG_LEVEL"
    ),
    audit_log: str = typer.Option(
        None, "--audit-log", help="Path to audit log file",
        envvar="TALOS_MCP_AUDIT_LOG_PATH"
    ),
    readonly: bool = typer.Option(
        False, "--readonly", help="Enable read-only mode (prevents mutating commands)",
        envvar="TALOS_MCP_READONLY"
    ),
) -> None:
    """Run the Talos MCP Server."""
    # Update global settings from CLI args
    settings.log_level = log_level
    if audit_log:
        settings.audit_log_path = audit_log
    settings.readonly = readonly

    configure_logging()
    uvloop.install()
    logger.info(f"Starting Talos MCP Server with log level {settings.log_level}")

    # Hint for users running interactively
    if sys.stdin.isatty():
        sys.stderr.write(
            "\n⚠️  This server expects JSON-RPC input from MCP clients (e.g., Claude Desktop).\n"
            "    Press Ctrl+C to exit.\n\n"
        )

    shutdown_event = asyncio.Event()

    def signal_handler() -> None:
        """Handle shutdown signals gracefully."""
        logger.info("Received shutdown signal, stopping server...")
        shutdown_event.set()

    async def run_server() -> None:
        # Setup signal handlers
        loop = asyncio.get_running_loop()
        for sig in (signal.SIGINT, signal.SIGTERM):
            loop.add_signal_handler(sig, signal_handler)

        try:
            async with stdio_server() as (read_stream, write_stream):
                # Run server with shutdown monitoring
                server_task = asyncio.create_task(
                    app_mcp.run(read_stream, write_stream, app_mcp.create_initialization_options())
                )
                shutdown_task = asyncio.create_task(shutdown_event.wait())

                done, pending = await asyncio.wait(
                    [server_task, shutdown_task],
                    return_when=asyncio.FIRST_COMPLETED
                )

                for task in pending:
                    task.cancel()
                    try:
                        await task
                    except asyncio.CancelledError:
                        pass

                logger.info("Server stopped by user")
        finally:
            # Cleanup signal handlers
            for sig in (signal.SIGINT, signal.SIGTERM):
                loop.remove_signal_handler(sig)

    try:
        asyncio.run(run_server())
    except KeyboardInterrupt:
        pass  # Already handled by signal handler
    except BaseException as e:
        # Handle ExceptionGroup (Python 3.11+) or regular exceptions
        # Filter out BrokenResourceError which is expected during shutdown
        if isinstance(e, anyio.BrokenResourceError):
            pass  # Expected during shutdown
        elif hasattr(e, 'exceptions'):
            # ExceptionGroup-like: filter out BrokenResourceError
            real_errors = [
                exc for exc in e.exceptions 
                if not isinstance(exc, anyio.BrokenResourceError)
            ]
            if real_errors:
                logger.exception(f"Server crashed: {e}")
                sys.exit(1)
        elif not isinstance(e, (SystemExit, KeyboardInterrupt)):
            logger.exception(f"Server crashed: {e}")
            sys.exit(1)


if __name__ == "__main__":
    cli()



"""Test logging and error handling by provoking a failure."""

import asyncio
import logging
import sys


# Add src to path
sys.path.append("src")

from talos_mcp.core.client import TalosClient
from talos_mcp.core.exceptions import TalosCommandError


# Configure logging to show everything
logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("test_logging")


async def main():
    client = TalosClient()

    logger.info("--- Testing successful command (should log debug) ---")
    # Using 'version --client' which should work locally without cluster
    try:
        await client.execute_talosctl(["version", "--client"])
    except Exception as e:
        logger.error(f"Version command failed: {e}")

    logger.info("\n--- Testing failed command (should log error) ---")
    # This command should fail
    try:
        await client.execute_talosctl(["invalid-command-xyz"])
        print("Unexpected success for invalid command!")
    except TalosCommandError as e:
        print(f"Correctly caught error: {e}")


if __name__ == "__main__":
    asyncio.run(main())



"""Verify that all tools are registered correctly."""

import asyncio
import logging
import sys


# Add src to path
sys.path.append("src")

from talos_mcp.server import list_tools


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("verify_tools")


async def main():
    tools = await list_tools()
    logger.info(f"Found {len(tools)} tools registered.")
    for tool in tools:
        print(f"- {tool.name}: {tool.description}")

    expected_tools = [
        "talos_version",
        "talos_health",
        "talos_stats",
        "talos_containers",
        "talos_processes",
        "talos_dashboard",
        "talos_memory",
        "talos_time",
        "talos_ls",
        "talos_cat",
        "talos_cp",
        "talos_du",
        "talos_mounts",
        "talos_interfaces",
        "talos_routes",
        "talos_netstat",
        "talos_pcap",
        "talos_service",
        "talos_logs",
        "talos_dmesg",
        "talos_events",
        "talos_reboot",
        "talos_shutdown",
        "talos_reset",
        "talos_upgrade",
        "talos_image",
        "talos_etcd_members",
        "talos_etcd_snapshot",
        "talos_etcd_alarm",
        "talos_etcd_defrag",
        "talos_config_info",
        "talos_kubeconfig",
        "talos_apply_config",
        "talos_validate_config",
        "talos_get",
        "talos_definitions",
    ]

    missing = [t for t in expected_tools if not any(ft.name == t for ft in tools)]

    if missing:
        logger.error(f"Missing tools: {missing}")
        sys.exit(1)

    logger.info("All expected tools are registered successfully.")


if __name__ == "__main__":
    asyncio.run(main())



"""Application settings."""

import os
import tempfile

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    """Application settings.
    
    All settings can be configured via environment variables with TALOS_MCP_ prefix.
    Example: TALOS_MCP_LOG_LEVEL=DEBUG
    """

    model_config = SettingsConfigDict(env_prefix="TALOS_MCP_")

    # Core settings
    talos_config_path: str | None = Field(default=None, description="Path to talosconfig")
    readonly: bool = Field(default=False, description="Read-only mode")

    # Logging settings
    log_level: str = Field(default="INFO", description="Logging level (DEBUG, INFO, WARNING, ERROR)")
    log_format: str = Field(
        default=(
            "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "
            "<level>{level: <8}</level> | "
            "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - "
            "<level>{message}</level>"
        ),
        description="Log format for console output"
    )

    # Audit log settings
    audit_log_path: str = Field(
        default_factory=lambda: os.path.join(tempfile.gettempdir(), "talos_mcp_audit.log"),
        description="Audit log file path"
    )
    audit_log_rotation: str = Field(default="10 MB", description="Audit log rotation size")
    audit_log_retention: str = Field(default="10 days", description="Audit log retention period")
    audit_log_serialize: bool = Field(default=True, description="Serialize audit logs as JSON")


settings = Settings()



"""Services management tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class ServiceSchema(BaseModel):
    """Schema for service arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    action: str = Field(default="status", description="Action: status, start, stop, restart")
    service: str | None = Field(default=None, description="Service name (optional for status)")


class ServiceTool(TalosTool):
    """Service operations."""

    name = "talos_service"
    description = "Manage services"
    args_schema = ServiceSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = ServiceSchema(**arguments)
        # talosctl service <id> --node <node> (for status)
        # talosctl service <id> start --node <node>
        # If action is status and no service, list all? `talosctl service` lists all.

        cmd = ["service"]
        if args.service:
            cmd.append(args.service)

        if args.action != "status":
            cmd.append(args.action)

        nodes = self.ensure_nodes(args.nodes)
        cmd.extend(["-n", nodes])
        return await self.execute_talosctl(cmd)


class LogsSchema(BaseModel):
    """Schema for logs arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    service: str = Field(description="Service name or container name")
    lines: int = Field(default=100, description="Number of lines to tail")
    follow: bool = Field(default=False, description="Follow logs")


class LogsTool(TalosTool):
    """Get service logs."""

    name = "talos_logs"
    description = "Get logs from services"
    args_schema = LogsSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = LogsSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["logs", args.service, "-n", nodes, "--tail", str(args.lines)]
        if args.follow:
            cmd.append("--follow")
        return await self.execute_talosctl(cmd)


class DmesgSchema(BaseModel):
    """Schema for dmesg arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    follow: bool = Field(default=False, description="Follow logs")


class DmesgTool(TalosTool):
    """Get kernel logs."""

    name = "talos_dmesg"
    description = "Get kernel logs (dmesg)"
    args_schema = DmesgSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = DmesgSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["dmesg", "-n", nodes]
        if args.follow:
            cmd.append("--follow")
        return await self.execute_talosctl(cmd)


class EventsSchema(BaseModel):
    """Schema for events arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")
    duration: str = Field(default="0s", description="Duration to stream events (0s = forever)")


class EventsTool(TalosTool):
    """Get events."""

    name = "talos_events"
    description = "Get system events"
    args_schema = EventsSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = EventsSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        cmd = ["events", "-n", nodes]
        # events streams forever by default.
        # We should probably limit it for MCP unless using SSE streaming properly.
        # But 'call_tool' expects a return. So we probably just want a snapshot or short duration?
        # Actually `talosctl events` streams.
        # For now, let's just run it. If it blocks, it blocks.
        if args.duration != "0s":
            cmd.extend(["--duration", args.duration])

        return await self.execute_talosctl(cmd)



"""System information tools."""

from typing import Any

from mcp.types import TextContent
from pydantic import BaseModel, Field

from talos_mcp.tools.base import TalosTool


class NodesSchema(BaseModel):
    """Schema for node arguments."""

    nodes: str | None = Field(default=None, description="Comma-separated list of node IPs/hostnames. Defaults to all nodes if not provided.")


class GetVersionTool(TalosTool):
    """Get version."""

    name = "talos_version"
    description = "Get Talos Linux version information from nodes"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["version", "-n", nodes])


class GetHealthTool(TalosTool):
    """Get health."""

    name = "talos_health"
    description = "Check health status of Talos cluster"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)

        # talosctl health does not support multiple nodes.
        # It's a cluster-wide check, so we just pick the first node as the endpoint.
        node_list = nodes.split(",")
        target_node = node_list[0]

        return await self.execute_talosctl(["health", "-n", target_node])


class GetStatsTool(TalosTool):
    """Get stats."""

    name = "talos_stats"
    description = "Get container stats (CPU/Memory usage) from nodes"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["stats", "-n", nodes])


class GetContainersTool(TalosTool):
    """Get containers."""

    name = "talos_containers"
    description = "List containers running on the node"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["containers", "-n", nodes])


class GetProcessesTool(TalosTool):
    """Get processes."""

    name = "talos_processes"
    description = "List processes running on the node"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["processes", "-n", nodes])


class DashboardTool(TalosTool):
    """Get dashboard."""

    name = "talos_dashboard"
    description = "Get a snapshot of the Talos dashboard"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        # Dashboard is interactive TUI. We can't really pipe it well unless we use specific flags.
        # It is not supported via MCP.
        # Actually `talosctl dashboard` is TUI.
        return [TextContent(type="text", text="Dashboard is a TUI and cannot be rendered in MCP.")]


class MemoryTool(TalosTool):
    """Get memory."""

    name = "talos_memory"
    description = "Get memory usage details"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["memory", "-n", nodes])


class TimeTool(TalosTool):
    """Get time."""

    name = "talos_time"
    description = "Get system time"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["time", "-n", nodes])


class DisksTool(TalosTool):
    """Get disks."""

    name = "talos_disks"
    description = "List disk drives and their properties"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["get", "disks", "-n", nodes])


class DevicesTool(TalosTool):
    """Get devices (PCI, USB, etc)."""

    name = "talos_devices"
    description = "List hardware devices (PCI, USB, System) via resource definitions"
    args_schema = NodesSchema

    async def run(self, arguments: dict[str, Any]) -> list[TextContent]:
        """Execute the tool."""
        args = NodesSchema(**arguments)
        nodes = self.ensure_nodes(args.nodes)
        return await self.execute_talosctl(["get", "devices", "-n", nodes])




"""MCP Resources implementation for Talos."""

from mcp.types import Resource, ResourceTemplate
from pydantic import AnyUrl

from talos_mcp.core.client import TalosClient
from talos_mcp.core.exceptions import TalosCommandError


class TalosResources:
    """Talos Resources handler."""

    def __init__(self, client: TalosClient) -> None:
        """Initialize TalosResources.

        Args:
            client: TalosClient instance
        """
        self.client = client

    async def list_resources(self) -> list[Resource]:
        """List available resources."""
        # Since resources are dynamic (per node), we use templates mostly.
        # But we can list some static ones if needed.
        return []

    async def list_resource_templates(self) -> list[ResourceTemplate]:
        """List available resource templates."""
        return [
            ResourceTemplate(
                uriTemplate="talos://{node}/health",
                name="Node Health",
                description="Get health status for a specific node",
                mimeType="text/plain",
            ),
            ResourceTemplate(
                uriTemplate="talos://{node}/version",
                name="Node Version",
                description="Get version information for a specific node",
                mimeType="text/plain",
            ),
            ResourceTemplate(
                uriTemplate="talos://{node}/config",
                name="Node Config Context",
                description="Get configuration context information",
                mimeType="text/plain",
            ),
        ]

    async def read_resource(self, uri: AnyUrl) -> str:
        """Read a Talos resource.

        Supported schemes:
        - talos://<node_ip>/health
        - talos://<node_ip>/version
        - talos://<node_ip>/config (Context info)
        """
        scheme = uri.scheme
        path = uri.path
        host = uri.host

        if scheme != "talos":
            raise ValueError(f"Unsupported scheme: {scheme}")

        if not host:
             # handle case where host might be empty or part of path depending on parsing
             pass

        # Normalize path
        resource_type = path.strip("/") if path else ""

        if resource_type == "health":
            return await self._get_health(host)
        elif resource_type == "version":
            return await self._get_version(host)
        elif resource_type == "config":
             # Config is cluster-wide usually, but we keep URI structure consistent
            return await self._get_config_info()
        else:
            raise ValueError(f"Unknown resource type: {resource_type}")

    async def _get_health(self, node: str | None) -> str:
        """Get health status for a node."""
        args = ["health"]
        if node:
            args.extend(["--nodes", node])
        try:
            result = await self.client.execute_talosctl(args)
            return str(result["stdout"])
        except TalosCommandError as e:
            return f"Error getting health: {e.stderr}"
        except Exception as e:
            return f"Error getting health: {e!s}"

    async def _get_version(self, node: str | None) -> str:
        """Get version info for a node."""
        args = ["version"]
        if node:
            args.extend(["--nodes", node])
        try:
            result = await self.client.execute_talosctl(args)
            return str(result["stdout"])
        except TalosCommandError as e:
            return f"Error getting version: {e.stderr}"
        except Exception as e:
            return f"Error getting version: {e!s}"

    async def _get_config_info(self) -> str:
        """Get config info."""
        # Using context info as a proxy for "config" resource
        info = self.client.get_context_info()
        return str(info)



#!/usr/bin/env python3
"""Test script for Talos MCP Server

This script verifies that the MCP server can communicate with your Talos cluster.
"""

import asyncio
import sys
from pathlib import Path


# Add src to path for imports
sys.path.insert(0, str(Path(__file__).parent / "src"))

from talos_mcp.core.client import TalosClient


async def test_connection():
    """Test connection to Talos cluster"""
    print("🔍 Testing Talos MCP Server Connection")
    print("=" * 50)
    print()

    # Initialize client
    client = TalosClient()

    # Test 1: Check config
    print("1️⃣  Checking Talos configuration...")
    config_info = client.get_context_info()

    if "error" in config_info:
        print(f"   ❌ {config_info['error']}")
        print()
        print("   Please ensure you have a valid talosconfig file.")
        print("   Run: talosctl config info")
        return False
    else:
        print("   ✅ Configuration loaded")
        print(f"      Context: {config_info.get('context', 'N/A')}")
        print(f"      Endpoints: {', '.join(config_info.get('endpoints', []))}")
        print()

    # Test 2: Check talosctl
    print("2️⃣  Testing talosctl command...")
    result = await client.execute_talosctl(["version", "--client"])

    if result["success"]:
        print("   ✅ talosctl is working")
        # Print first line of version output
        version_line = result["stdout"].split("\n")[0] if result["stdout"] else ""
        if version_line:
            print(f"      {version_line}")
        print()
    else:
        print(f"   ❌ talosctl failed: {result.get('stderr', 'Unknown error')}")
        print()
        return False

    # Test 3: Try to connect to cluster
    print("3️⃣  Testing cluster connection...")
    result = await client.execute_talosctl(["version"])

    if result["success"]:
        print("   ✅ Successfully connected to Talos cluster")
        print()
        # Print version info
        for line in result["stdout"].split("\n")[:5]:
            if line.strip():
                print(f"      {line}")
        print()
    else:
        print("   ⚠️  Could not connect to cluster")
        print(f"      {result.get('stderr', 'Unknown error')}")
        print()
        print("   This might be expected if your cluster is not running.")
        print("   The MCP server is configured correctly, but cluster is unreachable.")
        print()

    print("=" * 50)
    print("✅ Talos MCP Server is set up correctly!")
    print()
    print("You can now:")
    print("  1. Add this server to your Claude Desktop configuration")
    print("  2. Restart Claude Desktop")
    print("  3. Start asking Claude about your Talos cluster")
    print()
    return True


if __name__ == "__main__":
    try:
        success = asyncio.run(test_connection())
        sys.exit(0 if success else 1)
    except KeyboardInterrupt:
        print("\n\n⚠️  Test interrupted by user")
        sys.exit(1)
    except Exception as e:
        print(f"\n\n❌ Test failed with error: {e}")
        sys.exit(1)



"""Talos Client module."""

import asyncio
import os
import shutil
from pathlib import Path
from typing import Any

import arrow
import yaml
from loguru import logger
from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_exponential

from talos_mcp.core.exceptions import TalosCommandError
from talos_mcp.core.settings import settings


class TalosClient:
    """Client for interacting with Talos Linux API."""

    def __init__(self, config_path: str | None = None) -> None:
        """Initialize Talos client with configuration.

        Args:
            config_path: Path to talosconfig file. Defaults to ~/.talos/config.
        """
        self.config_path = (
            config_path
            or os.environ.get("TALOSCONFIG")
            or settings.talos_config_path
            or os.path.expanduser("~/.talos/config")
        )
        self.config: dict[str, Any] | None = None
        self.current_context: str | None = None
        self._load_config()

    def _load_config(self) -> None:
        """Load Talos configuration from file."""
        try:
            config_file = Path(self.config_path)
            if config_file.exists():
                with config_file.open() as f:
                    self.config = yaml.safe_load(f)
                    if self.config:
                        self.current_context = self.config.get("context")
                        logger.info(f"Loaded Talos config with context: {self.current_context}")
            else:
                logger.warning(f"Talos config not found at {self.config_path}")
        except Exception as e:
            logger.error(f"Error loading Talos config: {e}")

    def get_context_info(self) -> dict[str, Any]:
        """Get information about the current context.

        Returns:
            Dictionary with context information or error.
        """
        if not self.config or not self.current_context:
            return {"error": "No Talos configuration loaded"}

        contexts = self.config.get("contexts", {})
        context_data = contexts.get(self.current_context, {})

        return {
            "context": self.current_context,
            "target": context_data.get("target"),
            "endpoints": context_data.get("endpoints", []),
            "nodes": context_data.get("nodes", []),
            "config_path": self.config_path,
        }

    def get_nodes(self) -> list[str]:
        """Get all nodes configured in the current context.

        Returns:
            List of node IPs/hostnames.
        """
        if not self.config or not self.current_context:
            return []

        contexts = self.config.get("contexts", {})
        context_data = contexts.get(self.current_context, {})

        nodes = context_data.get("nodes", [])
        if nodes:
            return nodes

        # Fallback to endpoints if nodes are not explicitly set
        # Endpoints might contain ports (e.g. 1.2.3.4:6443), which we should strip for node addressing
        endpoints = context_data.get("endpoints", [])
        clean_nodes = []
        for ep in endpoints:
            if ":" in ep:
                ep = ep.split(":")[0]
            clean_nodes.append(ep)

        return clean_nodes

    @retry(
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=1, min=2, max=10),
        reraise=True,
        retry=retry_if_exception_type(
            (OSError, TimeoutError)
        ),  # Only retry on system/network errors, not command failures
    )
    async def execute_talosctl(self, args: list[str]) -> dict[str, str]:
        """Execute talosctl command and return the output.

        Args:
            args: List of arguments for talosctl.

        Returns:
            Dictionary containing stdout and stderr.

        Raises:
            TalosCommandError: If the command fails.
        """
        talosctl_path = shutil.which("talosctl")
        if not talosctl_path:
            raise TalosCommandError(["talosctl"], 127, "talosctl not found in PATH")

        cmd = [talosctl_path, *args]

        # Add config file if it exists
        if Path(self.config_path).exists():
            cmd.extend(["--talosconfig", self.config_path])

        # Check for read-only mode via settings
        if settings.readonly:
            # Basic protection using simple string matching
            unsafe_commands = [
                "upgrade",
                "reset",
                "reboot",
                "shutdown",
                "apply-config",
                "bootstrap",
                "defrag",
            ]
            if any(cmd_part in args for cmd_part in unsafe_commands):
                 raise TalosCommandError(cmd, 1, "Operation not permitted in read-only mode")

        start_time = arrow.now()
        logger.debug(f"Executing: {' '.join(cmd)} at {start_time.format()}")

        try:
            logger.debug(f"Executing command: {' '.join(cmd)}")
            process = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )

            stdout, stderr = await process.communicate()

            stdout_str = stdout.decode().strip()
            stderr_str = stderr.decode().strip()
            duration = arrow.now() - start_time

            logger.debug(f"Command finished in {duration.total_seconds():.2f}s")

            if process.returncode != 0:
                logger.error(
                    f"Command failed with code {process.returncode}\n"
                    f"Cmd: {' '.join(cmd)}\n"
                    f"Stderr: {stderr_str}"
                )
                raise TalosCommandError(cmd, process.returncode, stderr_str)
            else:
                logger.debug(f"Command success: {' '.join(cmd)}")

            return {
                "stdout": stdout_str,
                "stderr": stderr_str,
            }

        except TalosCommandError:
            raise
        except Exception as e:
            # Tenacity will handle retries for specific exceptions,
            # but if we exhaust them or hit others:
            logger.opt(exception=True).error(
                f"Error executing talosctl: {' '.join(cmd)}"
            )
            # Wrap unknown errors in TalosCommandError for consistency
            raise TalosCommandError(cmd, 1, f"Execution failed: {e!s}")
